var _default_vert = 
`
attribute vec4 a_position;
attribute vec2 a_texCoord;
attribute vec4 a_color;
varying vec2 v_texCoord;
varying vec4 v_fragmentColor;
void main()
{
    gl_Position = CC_PMatrix * a_position;
    v_fragmentColor = a_color;
    v_texCoord = a_texCoord;
}
`;
var _gray_frag =
`
#ifdef GL_ES
precision mediump float;
#endif
varying vec2 v_texCoord;
void main()
{
    vec4 v = texture2D(CC_Texture0, v_texCoord).rgba;
    float f = v.r * 0.299 + v.g * 0.587 + v.b * 0.114;
    gl_FragColor = vec4(f, f, f, v.a);
}
`;

var _black_white_frag = 
`
#ifdef GL_ES
precision mediump float;
#endif
varying vec2 v_texCoord;
void main()
{
    vec4 v = texture2D(CC_Texture0, v_texCoord).rgba;
    gl_FragColor = v;
}
`;

cc.Class({
    extends: cc.Component,

    properties: {
		childEnabled: true
    },
	
	editor: {
        requireComponent: cc.Sprite,
        executeInEditMode: true
    },

    onEnable: function () {
        this._use();
    },

    onDisable: function () {
        this._unuse();
    },

    onLoad: function () {
        this.setGray(false);
    },
    
    _use: function(){
        if( !this._program )
		    this._program = new cc.GLProgram();

        if (cc.sys.isNative) {
            //cc.log("use native GLProgram")
            this._program.initWithString(_default_vert, _gray_frag);
            this._program.link();
            this._program.updateUniforms();
        }else{
            this._program.initWithVertexShaderByteArray(_default_vert, _gray_frag);
            this._program.addAttribute(cc.macro.ATTRIBUTE_NAME_POSITION, cc.macro.VERTEX_ATTRIB_POSITION);
            this._program.addAttribute(cc.macro.ATTRIBUTE_NAME_COLOR, cc.macro.VERTEX_ATTRIB_COLOR);
            this._program.addAttribute(cc.macro.ATTRIBUTE_NAME_TEX_COORD, cc.macro.VERTEX_ATTRIB_TEX_COORDS);
            this._program.link();
            this._program.updateUniforms();
        }
        this.setProgram( this.node._sgNode, this._program );
    },
	
	_unuse: function(){
        if( !this._program )
		    this._program = new cc.GLProgram();

        if (cc.sys.isNative) {
            //cc.log("use native GLProgram")
            this._program.initWithString(_default_vert, _black_white_frag);
            this._program.link();
            this._program.updateUniforms();
        }else{
            this._program.initWithVertexShaderByteArray(_default_vert, _black_white_frag);
            this._program.addAttribute(cc.macro.ATTRIBUTE_NAME_POSITION, cc.macro.VERTEX_ATTRIB_POSITION);
            this._program.addAttribute(cc.macro.ATTRIBUTE_NAME_COLOR, cc.macro.VERTEX_ATTRIB_COLOR);
            this._program.addAttribute(cc.macro.ATTRIBUTE_NAME_TEX_COORD, cc.macro.VERTEX_ATTRIB_TEX_COORDS);
            this._program.link();
            this._program.updateUniforms();
        }
        this.setProgram( this.node._sgNode, this._program );
	},
	
    setProgram:function (node, program) {
        if (cc.sys.isNative) {
            var glProgram_state = cc.GLProgramState.getOrCreateWithGLProgram(program);
            node.setGLProgramState(glProgram_state);
        }else{
            node.setShaderProgram(program);    
        }
	
		if( this.childEnabled ){
			var children = node.children;
			if (!children)
				return;
			for (var i = 0; i < children.length; i++)
				this.setProgram(children[i], program);
		}
    },
	
	setGray: function(isGray){
		this.enabled = !!isGray;
	}
});
